package com.enx_plugin.enx_flutter_plugin;

import android.app.Activity;
import android.graphics.Bitmap;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import enx_rtc_android.Controller.EnxActiveTalkerListObserver;
import enx_rtc_android.Controller.EnxAdvancedOptionsObserver;
import enx_rtc_android.Controller.EnxBandwidthObserver;
import enx_rtc_android.Controller.EnxCanvasObserver;
import enx_rtc_android.Controller.EnxChairControlObserver;
import enx_rtc_android.Controller.EnxFileShareObserver;
import enx_rtc_android.Controller.EnxLockRoomManagementObserver;
import enx_rtc_android.Controller.EnxLogsObserver;
import enx_rtc_android.Controller.EnxMuteAudioStreamObserver;
import enx_rtc_android.Controller.EnxMuteRoomObserver;
import enx_rtc_android.Controller.EnxMuteVideoStreamObserver;
import enx_rtc_android.Controller.EnxNetworkObserever;
import enx_rtc_android.Controller.EnxOutBoundCallObserver;
import enx_rtc_android.Controller.EnxPlayerView;
import enx_rtc_android.Controller.EnxReconnectObserver;
import enx_rtc_android.Controller.EnxRecordingObserver;
import enx_rtc_android.Controller.EnxRoom;
import enx_rtc_android.Controller.EnxRoomObserver;
import enx_rtc_android.Controller.EnxRtc;
import enx_rtc_android.Controller.EnxScreenShareObserver;
import enx_rtc_android.Controller.EnxScreenShotObserver;
import enx_rtc_android.Controller.EnxStatsObserver;
import enx_rtc_android.Controller.EnxStream;
import enx_rtc_android.Controller.EnxStreamObserver;
import enx_rtc_android.Controller.EnxTalkerObserver;
import enx_rtc_android.Controller.EnxUtilityManager;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.StandardMessageCodec;

import static com.enx_plugin.enx_flutter_plugin.EnxUtils.convertMapToJson;
import static com.enx_plugin.enx_flutter_plugin.EnxUtils.jsonToMap;
import static com.enx_plugin.enx_flutter_plugin.EnxUtils.toList;

/**
 * EnxFlutterPlugin
 */
public class EnxFlutterPlugin implements FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    //
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;

    /**
     * The EventChannel used to pass observer callbacks to flutter plugin
     */
    private EventChannel eventChannel;
    private Handler mEventHandler = new Handler(Looper.getMainLooper());
    private EventChannel.EventSink sink;
    private EnxRtc enxRtc;
    private EnxStream localStream;
    private HashMap<String, FrameLayout> mRendererViews;
    private EnxRoom mEnxRoom;
    public Activity mActivity;

    /**
     * map for handling remote streams
     */
    private ConcurrentHashMap<String, EnxStream> mRemoteStream = new ConcurrentHashMap<>();

    /**
     * map for handling activetalker streams
     */
    private ConcurrentHashMap<String, EnxStream> mActiveStreams = new ConcurrentHashMap<>();

    void addView(FrameLayout view, int id) {
        mRendererViews.put("" + id, view);
    }


    public ConcurrentHashMap<String, EnxStream> getRemoteStream() {

        return this.mRemoteStream;
    }

    public ConcurrentHashMap<String, EnxStream> getActiveStream() {

        return this.mActiveStreams;
    }

    private void removeView(int id) {
        mRendererViews.remove("" + id);
    }

    FrameLayout getView(int id) {
        return mRendererViews.get("" + id);
    }

    /**
     * Plugin registration.
     */
    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "enx_flutter_plugin");
        eventChannel = new EventChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "enx_flutter_plugin_event_channel");
        this.sink = null;
        this.mRendererViews = new HashMap<>();
        channel.setMethodCallHandler(this);
        eventChannel.setStreamHandler(this);
        EnxPlayerViewFactory fac = new EnxPlayerViewFactory(StandardMessageCodec.INSTANCE,
                this);
        flutterPluginBinding.getPlatformViewRegistry().registerViewFactory("EnxPlayer", fac);
    }

    /**
     * used for taking methods and parameters from flutter Plugin
     */
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        Log.d("onMethodCall", call.method);
        String clientId = "";
        int viewId = -1;
        int uid = -1;
        boolean isLocal = false;
        if (call.hasArgument("clientId")) {
            clientId = call.argument("clientId");
        }
        if (call.hasArgument("viewId")) {
            viewId = call.argument("viewId");
        }
        if (call.hasArgument("uid")) {
            uid = call.argument("uid");
        }
        if (call.hasArgument("isLocal")) {
            isLocal = call.argument("isLocal");
        }
        switch (call.method) {
            case "joinRoom":
                String token = call.argument("token");
                HashMap<String, Object> localInfo;
                HashMap<String, Object> roomInfo;
                ArrayList<Object> advanceOptions;

                JSONObject localInfoObject = null;
                JSONObject roomInfoObject = null;
                JSONObject advanceOptionsObject = null;
                try {
                    if (call.argument("localInfo") != null) {
                        localInfo = call.argument("localInfo");
                        localInfoObject = convertMapToJson(localInfo);
                    }
                    if (call.argument("roomInfo") != null) {
                        roomInfo = call.argument("roomInfo");
                        roomInfoObject = convertMapToJson(roomInfo);
                    }
                    if (call.argument("advanceOptions") != null) {
                        advanceOptions = call.argument("advanceOptions");
                        advanceOptionsObject = getAdvancedOptionsObject(advanceOptions);
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                localStream = enxRtc.joinRoom(token, localInfoObject, getRoomInfo(), advanceOptionsObject);
                if (localStream != null) {
                    localStream.setMuteAudioStreamObserver(mEnxMuteAudioStreamObserver);
                    localStream.setMuteVideoStreamObserver(mEnxMuteVideoStreamObserver);
                }
                result.success(null);
                break;
            case "removeNativeView":
                removeView(viewId);
                result.success(null);
                break;

            case "publish":
                if (mEnxRoom != null) {
                    mEnxRoom.publish(localStream);
                }
                result.success(null);
                break;

            case "disconnect":
                if (mEnxRoom != null) {
                    mEnxRoom.disconnect();
                }
                result.success(null);
                break;
            case "startRecord":
                if (mEnxRoom != null) {
                    mEnxRoom.startRecord();
                }
                result.success(null);
                break;
            case "stopRecord":
                if (mEnxRoom != null) {
                    mEnxRoom.stopRecord();
                }
                result.success(null);
                break;
            case "setTalkerCount":
                int count = call.argument("count");
                if (mEnxRoom != null) {
                    mEnxRoom.setTalkerCount(count);
                }
                result.success(null);
                break;
            case "getTalkerCount":
                if (mEnxRoom != null) {
                    mEnxRoom.getTalkerCount();
                }
                result.success(null);
                break;
            case "getMaxTalkers":
                if (mEnxRoom != null) {
                    mEnxRoom.getMaxTalkers();
                }
                result.success(null);
                break;
            case "enableLogs":
                boolean status = call.argument("status");
                EnxUtilityManager enxLogsUtil = EnxUtilityManager.getInstance();
                enxLogsUtil.enableLogs(status);
                result.success(null);
                break;
            case "postClientLogs":
                if (mEnxRoom != null) {
                    mEnxRoom.postClientLogs();
                }
                result.success(null);
                break;
            case "requestFloor":
                if (mEnxRoom != null) {
                    mEnxRoom.requestFloor();
                }
                result.success(null);
                break;
            case "grantFloor":
                if (mEnxRoom != null) {
                    mEnxRoom.grantFloor(clientId);
                }
                result.success(null);
                break;
            case "cancelFloor":
                if (mEnxRoom != null) {
                    mEnxRoom.cancelFloor();
                }
                result.success(null);
                break;
            case "finishFloor":
                if (mEnxRoom != null) {
                    mEnxRoom.finishFloor();
                }
                result.success(null);
                break;
            case "denyFloor":
                if (mEnxRoom != null) {
                    mEnxRoom.denyFloor(clientId);
                }
                result.success(null);
                break;
            case "releaseFloor":
                if (mEnxRoom != null) {
                    mEnxRoom.releaseFloor(clientId);
                }
                result.success(null);
                break;
            case "hardMute":
                if (mEnxRoom != null) {
                    mEnxRoom.hardMute();
                }
                result.success(null);
                break;
            case "hardUnMute":
                if (mEnxRoom != null) {
                    mEnxRoom.hardUnMute();
                }
                result.success(null);
                break;
            case "muteSelfAudio":
                boolean isMuteAudio = call.argument("isMute");
                if (mEnxRoom != null) {
                    localStream.muteSelfAudio(isMuteAudio);
                }
                result.success(null);
                break;
            case "muteSelfVideo":
                boolean isMuteVideo = call.argument("isMute");
                if (mEnxRoom != null) {
                    localStream.muteSelfVideo(isMuteVideo);
                }
                result.success(null);
                break;

            case "enableProximitySensor":
                boolean isEnabled = call.argument("isEnabled");
                if (mEnxRoom != null) {
                    mEnxRoom.enableProximitySensor(isEnabled);
                }
                result.success(null);
                break;
            case "hardMuteAudio":
                if (mEnxRoom != null) {
                    localStream.hardMuteAudio(clientId);
                }
                result.success(null);
                break;
            case "hardUnMuteAudio":
                if (mEnxRoom != null) {
                    localStream.hardUnMuteAudio(clientId);
                }
                result.success(null);
                break;
            case "hardMuteVideo":
                if (mEnxRoom != null) {
                    localStream.hardMuteVideo(clientId);
                }
                result.success(null);
                break;
            case "hardUnMuteVideo":
                if (mEnxRoom != null) {
                    localStream.hardUnMuteVideo(clientId);
                }
                result.success(null);
                break;
            case "switchCamera":
                if (localStream != null) {
                    localStream.switchCamera();
                }
                result.success(null);
                break;
            case "getAdvancedOptions":
                if (mEnxRoom != null) {
                    mEnxRoom.getAdvancedOptions();
                }
                result.success(null);
                break;
            case "sendData":
                if (mEnxRoom != null) {
                    String data = call.argument("data");
                    JSONObject dataObject = null;
                    try {
                        if (data != null) {
                            dataObject = new JSONObject(data);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    localStream.sendData(dataObject);
                }
                result.success(null);
                break;
            case "lockRoom":
                if (mEnxRoom != null) {
                    mEnxRoom.lockRoom();
                }
                result.success(null);
                break;
            case "unLockRoom":
                if (mEnxRoom != null) {
                    mEnxRoom.unLockRoom();
                }
                result.success(null);
                break;
            case "makeOutboundCall":
                String number = call.argument("number");
                if (mEnxRoom != null) {
                    mEnxRoom.makeOutboundCall(number);
                }
                result.success(null);
                break;
            case "extendConferenceDuration":
                if (mEnxRoom != null) {
                    mEnxRoom.extendConferenceDuration();
                }
                result.success(null);
                break;
            case "destroy":
                if (mEnxRoom != null) {
                    mEnxRoom.destroy();
                }
                result.success(null);
                break;
            case "stopVideoTracksOnApplicationBackground":
                boolean videoMuteLocalStream = call.argument("videoMuteLocalStream");
                boolean videoMuteRemoteStream = call.argument("videoMuteRemoteStream");
                if (mEnxRoom != null) {
                    mEnxRoom.stopVideoTracksOnApplicationBackground(videoMuteRemoteStream, videoMuteLocalStream);
                }
                result.success(null);
                break;
            case "startVideoTracksOnApplicationForeground":
                boolean restoreVideoRemoteStream = call.argument("restoreVideoRemoteStream");
                boolean restoreVideoLocalStream = call.argument("restoreVideoLocalStream");
                if (mEnxRoom != null) {
                    mEnxRoom.startVideoTracksOnApplicationForeground(restoreVideoRemoteStream, restoreVideoLocalStream);
                }
                result.success(null);
                break;
            case "setAudioOnlyMode":
                boolean audioOnly = call.argument("audioOnly");
                if (mEnxRoom != null) {
                    mEnxRoom.setAudioOnlyMode(audioOnly);
                }
                result.success(null);
                break;
            case "switchMediaDevice":
                String deviceName = call.argument("deviceName");
                if (mEnxRoom != null) {
                    mEnxRoom.switchMediaDevice(deviceName);
                }
                result.success(null);
                break;
            case "switchUserRole":
                if (mEnxRoom != null) {
                    mEnxRoom.switchUserRole(clientId);
                }
                result.success(null);
                break;
            case "getAvailableFiles":
                if (mEnxRoom != null) {
                    try {
                        if (mEnxRoom.getAvailableFiles() != null) {
                            result.success(jsonToMap(mEnxRoom.getAvailableFiles()));
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
            case "getDevices":
                if (mEnxRoom != null) {
                    try {
                        if (mEnxRoom.getDevices() != null) {
                            result.success(mEnxRoom.getDevices());
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                break;
            case "enableStats":
                boolean enableStats = call.argument("enableStats");
                if (mEnxRoom != null) {
                    mEnxRoom.enableStats(enableStats, mEnxStatsObserver);
                }
                result.success(null);
                break;
            case "muteSubscribeStreamsAudio":
                boolean isMute = call.argument("isMute");
                if (mEnxRoom != null) {
                    mEnxRoom.muteSubscribeStreamsAudio(isMute);
                }
                result.success(null);
                break;
            case "getRoomId":
                if (mEnxRoom != null) {
                    if (mEnxRoom.getRoomId() != null) {
                        result.success(mEnxRoom.getRoomId());
                    }
                }
                break;
            case "getClientName":
                if (mEnxRoom != null) {
                    if (mEnxRoom.getClientId() != null) {
                        result.success(mEnxRoom.getClientName());
                    }
                }
                break;
            case "getClientId":
                if (mEnxRoom != null) {
                    if (mEnxRoom.getClientId() != null) {
                        result.success(mEnxRoom.getClientId());
                    }
                }
                break;
            case "getRole":
                if (mEnxRoom != null) {
                    if (mEnxRoom.getRole() != null) {
                        result.success(mEnxRoom.getRole());
                    }
                }
                break;
            case "isRoomActiveTalker":
                if (mEnxRoom != null) {
                    result.success(mEnxRoom.isRoomActiveTalker());
                }
                break;
            case "cancelDownload":
                int jobId = call.argument("jobId");
                if (mEnxRoom != null) {
                    mEnxRoom.cancelDownload(jobId);
                }
                result.success(null);
                break;
            case "cancelUpload":
                int upJobId = call.argument("jobId");
                if (mEnxRoom != null) {
                    mEnxRoom.cancelUpload(upJobId);
                }
                result.success(null);
                break;
            case "cancelAllDownloads":
                if (mEnxRoom != null) {
                    mEnxRoom.cancelAllDownloads();
                }
                result.success(null);
                break;
            case "cancelAllUploads":
                if (mEnxRoom != null) {
                    mEnxRoom.cancelAllUploads();
                }
                result.success(null);
                break;
            case "downloadFile":
                boolean isAutoSave = call.argument("autoSave");
                HashMap<String, Object> fileInfo;
                JSONObject fileInfoObject = null;
                try {
                    if (call.argument("file") != null) {
                        fileInfo = call.argument("file");
                        fileInfoObject = convertMapToJson(fileInfo);
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                if (mEnxRoom != null) {
                    mEnxRoom.downloadFile(fileInfoObject, isAutoSave);
                }
                result.success(null);
                break;
            case "updateConfiguration":
                HashMap<String, Object> configInfo;
                JSONObject configObject = null;
                try {
                    if (call.argument("config") != null) {
                        configInfo = call.argument("config");
                        configObject = convertMapToJson(configInfo);
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                if (localStream != null) {
                    localStream.updateConfiguration(configObject);
                }
                result.success(null);
                break;
            case "getSelectedDevice":
                if (mEnxRoom != null) {
                    if (mEnxRoom.getSelectedDevice() != null) {
                        result.success(mEnxRoom.getSelectedDevice());
                    }
                }
                break;
            case "sendUserData":
                JSONObject messageObject = null;
                HashMap<String, Object> messageMap;
                boolean isBroadCast = call.argument("isBroadCast");
                ArrayList<String> list = (ArrayList<String>) call.argument("recipientIDs");
                try {
                    if (call.argument("message") != null) {
                        messageMap = call.argument("message");
                        messageObject = convertMapToJson(messageMap);
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                if (mEnxRoom != null) {
                    mEnxRoom.sendUserData(messageObject, isBroadCast, list);
                }
                result.success(null);
                break;
            case "sendMessage":
                String message = call.argument("message");
                boolean broadCast = call.argument("isBroadCast");
                ArrayList<String> recipientIDs = (ArrayList<String>) call.argument("recipientIDs");
                if (mEnxRoom != null) {
                    mEnxRoom.sendMessage(message, broadCast, recipientIDs);
                }
                result.success(null);
                break;
            case "sendFiles":
                boolean broadCastStatus = call.argument("isBroadCast");
                ArrayList<String> clientList = (ArrayList<String>) call.argument("recipientIDs");
                if (mEnxRoom != null) {
                    mEnxRoom.sendFiles(broadCastStatus, clientList, mActivity);
                }
                result.success(null);
                break;
            case "getUserList":
                try {
                    result.success(EnxUtils.toList(mEnxRoom.getUserList()));
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                break;
            case "whoAmI":
                if (mEnxRoom != null) {
                    try {
                        result.success(jsonToMap(mEnxRoom.whoAmI()));
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
                break;
            case "setupVideo":
                if (isLocal) {
                    if (localStream != null) {
                        if (getView(0).getChildCount() > 0) {
                            getView(0).removeAllViews();
                        }
                        localStream.mEnxPlayerView.setZOrderMediaOverlay(true);
                        getView(0).addView(localStream.mEnxPlayerView);
                    }
                } else {

                    ConcurrentHashMap<String, EnxStream> remoteStream = getActiveStream();
//                    if(getActiveStream().size()>0){
                    Log.e("activeViewId:" + String.valueOf(viewId), "uid" + String.valueOf(uid));
                    if (getView(viewId) != null && remoteStream.get(String.valueOf(uid)) != null) {

                        if (getView(viewId).getChildCount() > 0) {
                            getView(viewId).removeAllViews();
                        }
//                        getView(viewId).removeAllViews();
                        getView(viewId).addView(remoteStream.get(String.valueOf(uid)).mEnxPlayerView);
                    }
//                    }
                }
                break;
            case "getReceiveVideoQuality":
                String streamType = call.argument("streamType");
                if (mEnxRoom != null) {
                    result.success(mEnxRoom.getReceiveVideoQuality(streamType));
                }
                break;
            case "dropUser":
                ArrayList<String> clientIds = (ArrayList<String>) call.argument("clientIds");
                if (mEnxRoom != null) {
                    mEnxRoom.dropUser(clientIds);
                }
                result.success(null);
                break;
            case "setAdvancedOptions":
                JSONArray array = call.argument("advanceOptions");
                if (mEnxRoom != null) {
                    mEnxRoom.setAdvancedOptions(array, mEnxAdvanceOptionObserver);
                }
                break;
            case "subscribe":
                String streamId = call.argument("streamId");
                ConcurrentHashMap<String, EnxStream> mSubscriberStreams = getRemoteStream();
                if (mEnxRoom != null) {
                    mEnxRoom.subscribe(mSubscriberStreams.get(streamId));
                }
                result.success(null);
                break;
            case "setPlayerScalingType":
                String type = call.argument("type");
                EnxPlayerView.ScalingType scalingType = EnxPlayerView.ScalingType.SCALE_ASPECT_BALANCED;
                if (type.equalsIgnoreCase("SCALE_ASPECT_BALANCED")) {
                    scalingType = EnxPlayerView.ScalingType.SCALE_ASPECT_BALANCED;
                } else if (type.equalsIgnoreCase("SCALE_ASPECT_FIT")) {
                    scalingType = EnxPlayerView.ScalingType.SCALE_ASPECT_FIT;
                } else if (type.equalsIgnoreCase("SCALE_ASPECT_FILL")) {
                    scalingType = EnxPlayerView.ScalingType.SCALE_ASPECT_FILL;
                }
                if (isLocal) {
                    if (localStream != null) {
//                        mRendererViews.get("0").setScalingType(scalingType);
//                        localStream.attachRenderer(mRendererViews.get("0"));
                    }
                } else {
                    ConcurrentHashMap<String, EnxStream> remoteStream = getRemoteStream();
                    if (remoteStream != null && remoteStream.size() > 0) {
//                        mRendererViews.get(String.valueOf(viewId)).setScalingType(scalingType);
//                        remoteStream.get(String.valueOf(uid)).attachRenderer(mRendererViews.get(String.valueOf(viewId)));
                    }
                }
                break;
            case "setZOrderMediaOverlay":
                boolean mediaOverlay = call.argument("mediaOverlay");
                if (isLocal) {
                    if (localStream != null) {
                        localStream.mEnxPlayerView.setZOrderMediaOverlay(mediaOverlay);
//                        mRendererViews.get("0").setZOrderMediaOverlay(mediaOverlay);
                    }
                } else {
                    ConcurrentHashMap<String, EnxStream> remoteStream = getRemoteStream();
                    if (remoteStream != null && remoteStream.size() > 0) {
//                        mRendererViews.get(String.valueOf(viewId)).setZOrderMediaOverlay(mediaOverlay);
                    }
                }

                break;

            default:
                result.notImplemented();
        }
    }

    /**
     * Plugin removed.
     */
    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
        eventChannel.setStreamHandler(null);
    }

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        this.sink = events;
    }

    @Override
    public void onCancel(Object arguments) {
        this.sink = null;
    }

    /**
     * called when activty binding done
     */
    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        Log.d("onAttachedToActivity", "Plugin");
        enxRtc = new EnxRtc(binding.getActivity(), mRoomObserver, mStreamObserver, mEnxAdvanceOptionObserver, mScreenShotObserver);
        mActivity = binding.getActivity();
    }

    /**
     * called when configuration changed for the activity
     */
    @Override
    public void onDetachedFromActivityForConfigChanges() {
        Log.d("onDetachedFromActivity", "Plugin");
    }

    /**
     * called when activity again attached after configuration changes
     */
    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
        Log.d("onReattachedToActivity", "Plugin");
    }

    /**
     * called when activity detached
     */
    @Override
    public void onDetachedFromActivity() {
        Log.d("onDetachedFromActivity", "Plugin");
    }

    /**
     * listobserver for adding the active talker streams
     */
    private final EnxActiveTalkerListObserver mListObserver = new EnxActiveTalkerListObserver() {
        @Override
        public void onActiveTalkerList(List<EnxStream> list) {
            getActiveStream().clear();
            if (list.size() == 0) {
                return;
            }
            ConcurrentHashMap<String, EnxStream> mSubscriberStreams = getActiveStream();
            for (int i = 0; i < list.size(); i++) {
                mSubscriberStreams.put(list.get(i).getId(), list.get(i));
            }


//
//            for (int i = 1; i < list.size(); i++) {
//                if(getView(i)==null){
//                    return;
//                }
//                if(getView(i).getChildCount()>0){
//                    getView(i).removeAllViews();
//                    getView(i).addView(list.get(i-1).mEnxPlayerView);
//                }else{
//                    getView(i).addView(list.get(i-1).mEnxPlayerView);
//                }
//            }

        }
    };

    /**
     * EnxRoomObserver for sending the room callbacks events
     */
    private final EnxRoomObserver mRoomObserver = new EnxRoomObserver() {
        @Override
        public void onRoomConnected(EnxRoom enxRoom, JSONObject jsonObject) {
            mEnxRoom = enxRoom;
            try {
                sendEvent("onRoomConnected", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mEnxRoom.setRecordingObserver(mRecordingObserver);
            mEnxRoom.setTalkerObserver(mEnxTalkerObserver);
            mEnxRoom.setLogsObserver(mEnxLogsObserver);
            mEnxRoom.setBandwidthObserver(mEnxBandwidthObserver);
            mEnxRoom.setNetworkChangeObserver(mEnxNetworkObserver);
            mEnxRoom.setChairControlObserver(mEnxChairControlObserver);
            mEnxRoom.setMuteRoomObserver(mEnxMuteRoomObserver);
            mEnxRoom.setLockRoomManagementObserver(mLockRoomObserver);
            mEnxRoom.setOutBoundCallObserver(mEnxOutBoundCallObserver);
            mEnxRoom.setReconnectObserver(mEnxReconnectObserver);
            mEnxRoom.setScreenShareObserver(mEnxScreenShareObserver);
            mEnxRoom.setCanvasObserver(mEnxCanvasObserver);
            mEnxRoom.setFileShareObserver(mEnxFileShareObserver);
//            mEnxRoom.setActiveTalkerViewObserver(mViewObserver);
            mEnxRoom.setActiveTalkerListObserver(mListObserver);
        }

        @Override
        public void onRoomError(JSONObject jsonObject) {
            try {
                sendEvent("onRoomError", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onUserConnected(JSONObject jsonObject) {
            try {
                sendEvent("onUserConnected", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onUserDisConnected(JSONObject jsonObject) {
            try {
                sendEvent("onUserDisConnected", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onPublishedStream(EnxStream enxStream) {
            HashMap<String, String> map = new HashMap<>();
            map.put("result", "0");
            map.put("msg", "The stream has been published.");
            if (localStream != null) {
                map.put("streamId", mEnxRoom.getLocalStreamID());
            }
//            if (localStream != null) {
//                getView(0).addView(localStream.mEnxPlayerView);
//            }
            sendEvent("onPublishedStream", map);

        }

        @Override
        public void onUnPublishedStream(EnxStream enxStream) {

        }

        @Override
        public void onStreamAdded(EnxStream enxStream) {
            ConcurrentHashMap<String, EnxStream> mSubscriberStreams = getRemoteStream();
            mSubscriberStreams.put(enxStream.getId(), enxStream);
            HashMap<String, Object> map = new HashMap<>();
            map.put("streamId", enxStream.getId());
            map.put("hasScreen", enxStream.hasScreen());
            map.put("hasData", enxStream.hasData());
            sendEvent("onStreamAdded", map);
        }

        @Override
        public void onSubscribedStream(EnxStream enxStream) {
//            ConcurrentHashMap<String, EnxStream> mSubscriberStreams = enxState.getRemoteStream();
//            mSubscriberStreams.put(enxStream.getId(), enxStream);
            HashMap<String, Object> map = new HashMap<>();
            map.put("result", "0");
            map.put("streamId", enxStream.getId());
            map.put("msg", "Stream subscribed successfully.");
            sendEvent("onSubscribedStream", map);
        }

        @Override
        public void onUnSubscribedStream(EnxStream enxStream) {

        }

        @Override
        public void onRoomDisConnected(JSONObject jsonObject) {
            try {
                sendEvent("onRoomDisConnected", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onActiveTalkerList(JSONObject jsonObject) {
            try {
                sendEvent("onActiveTalkerList", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onEventError(JSONObject jsonObject) {
            try {
                sendEvent("onEventError", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onEventInfo(JSONObject jsonObject) {
            try {
                sendEvent("onEventInfo", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onNotifyDeviceUpdate(String s) {
            try {
                HashMap<String, Object> map = new HashMap<>();
                map.put("msg", s);
                sendEvent("onNotifyDeviceUpdate", map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onAcknowledgedSendData(JSONObject jsonObject) {
            try {
                sendEvent("onAcknowledgedSendData", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onMessageReceived(JSONObject jsonObject) {
            try {
                sendEvent("onMessageReceived", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onUserDataReceived(JSONObject jsonObject) {
            try {
                sendEvent("onUserDataReceived", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onSwitchedUserRole(JSONObject jsonObject) {
            try {
                sendEvent("onSwitchedUserRole", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onUserRoleChanged(JSONObject jsonObject) {
            try {
                sendEvent("onUserRoleChanged", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onConferencessExtended(JSONObject jsonObject) {
            try {
                sendEvent("onConferencessExtended", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onConferenceRemainingDuration(JSONObject jsonObject) {
            try {
                sendEvent("onConferenceRemainingDuration", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onAckDropUser(JSONObject jsonObject) {
            try {
                sendEvent("onAckDropUser", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onAckDestroy(JSONObject jsonObject) {
            try {
                sendEvent("onAckDestroy", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    private final EnxScreenShotObserver mScreenShotObserver = new EnxScreenShotObserver() {
        @Override
        public void OnCapturedView(Bitmap bitmap) {
            try {
                HashMap<String, Object> map = new HashMap<>();
                map.put("bitmap", bitmap.toString());
                sendEvent("OnCapturedView", map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxStreamObserver for sending the stream callbacks events
     */
    private final EnxStreamObserver mStreamObserver = new EnxStreamObserver() {
        @Override
        public void onAudioEvent(JSONObject jsonObject) {
            try {
                sendEvent("onAudioEvent", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onVideoEvent(JSONObject jsonObject) {
            try {
                sendEvent("onVideoEvent", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedData(JSONObject jsonObject) {
            try {
                sendEvent("onReceivedData", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onRemoteStreamAudioMute(JSONObject jsonObject) {
            try {
                sendEvent("onRemoteStreamAudioMute", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onRemoteStreamAudioUnMute(JSONObject jsonObject) {
            try {
                sendEvent("onRemoteStreamAudioUnMute", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onRemoteStreamVideoMute(JSONObject jsonObject) {
            try {
                sendEvent("onRemoteStreamVideoMute", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onRemoteStreamVideoUnMute(JSONObject jsonObject) {
            try {
                sendEvent("onRemoteStreamVideoUnMute", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxRecordingObserver for sending the recording callbacks events
     */
    private final EnxRecordingObserver mRecordingObserver = new EnxRecordingObserver() {
        @Override
        public void onStartRecordingEvent(JSONObject jsonObject) {
            try {
                sendEvent("onStartRecordingEvent", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onRoomRecordingOn(JSONObject jsonObject) {
            try {
                sendEvent("onRoomRecordingOn", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onStopRecordingEvent(JSONObject jsonObject) {
            try {
                sendEvent("onStopRecordingEvent", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onRoomRecordingOff(JSONObject jsonObject) {
            try {
                sendEvent("onRoomRecordingOff", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxTalkerObserver for sending the talker callbacks events
     */
    private final EnxTalkerObserver mEnxTalkerObserver = new EnxTalkerObserver() {
        @Override
        public void onSetTalkerCount(JSONObject jsonObject) {
            try {
                sendEvent("onSetTalkerCount", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onGetTalkerCount(JSONObject jsonObject) {
            try {
                sendEvent("onGetTalkerCount", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onMaxTalkerCount(JSONObject jsonObject) {
            try {
                sendEvent("onMaxTalkerCount", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxLogsObserver for sending the logs callbacks events
     */
    private final EnxLogsObserver mEnxLogsObserver = new EnxLogsObserver() {
        @Override
        public void onLogUploaded(JSONObject jsonObject) {
            try {
                sendEvent("onLogUploaded", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxAdvancedOptionsObserver for sending advanced options callbacks events
     */
    private final EnxAdvancedOptionsObserver mEnxAdvanceOptionObserver = new EnxAdvancedOptionsObserver() {
        @Override
        public void onAdvancedOptionsUpdate(JSONObject jsonObject) {
            try {
                sendEvent("onAdvancedOptionsUpdate", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onGetAdvancedOptions(JSONObject jsonObject) {
            try {
                sendEvent("onGetAdvancedOptions", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxBandwidthObserver for sending bandwidth callbacks events
     */
    private final EnxBandwidthObserver mEnxBandwidthObserver = new EnxBandwidthObserver() {
        @Override
        public void onBandWidthUpdated(JSONArray jsonArray) {
            try {
                HashMap<String, Object> map = new HashMap<>();
                map.put("bandwidth", toList(jsonArray));
                sendEvent("onBandWidthUpdated", map);
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onShareStreamEvent(JSONObject jsonObject) {
            try {
                sendEvent("onShareStreamEvent", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onCanvasStreamEvent(JSONObject jsonObject) {
            try {
                sendEvent("onCanvasStreamEvent", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxNetworkObserever for sending network callbacks events
     */
    private final EnxNetworkObserever mEnxNetworkObserver = new EnxNetworkObserever() {
        @Override
        public void onConnectionInterrupted(JSONObject jsonObject) {
            try {
                sendEvent("onConnectionInterrupted", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onConnectionLost(JSONObject jsonObject) {
            try {
                sendEvent("onConnectionLost", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxChairControlObserver for sending chaircontrol callbacks events
     */
    private final EnxChairControlObserver mEnxChairControlObserver = new EnxChairControlObserver() {
        @Override
        public void onFloorRequested(JSONObject jsonObject) {
            try {
                sendEvent("onFloorRequested", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFloorRequestReceived(JSONObject jsonObject) {
            try {
                sendEvent("onFloorRequestReceived", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onProcessFloorRequested(JSONObject jsonObject) {
            try {
                sendEvent("onProcessFloorRequested", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onGrantedFloorRequest(JSONObject jsonObject) {
            try {
                sendEvent("onGrantedFloorRequest", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onDeniedFloorRequest(JSONObject jsonObject) {
            try {
                sendEvent("onDeniedFloorRequest", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReleasedFloorRequest(JSONObject jsonObject) {
            try {
                sendEvent("onReleasedFloorRequest", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFloorCancelled(JSONObject jsonObject) {
            try {
                sendEvent("onFloorCancelled", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFloorFinished(JSONObject jsonObject) {
            try {
                sendEvent("onFloorFinished", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onCancelledFloorRequest(JSONObject jsonObject) {
            try {
                sendEvent("onCancelledFloorRequest", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFinishedFloorRequest(JSONObject jsonObject) {
            try {
                sendEvent("onFinishedFloorRequest", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxMuteRoomObserver for sending mute/unmute callbacks events
     */
    private final EnxMuteRoomObserver mEnxMuteRoomObserver = new EnxMuteRoomObserver() {
        @Override
        public void onHardMuted(JSONObject jsonObject) {
            try {
                sendEvent("onHardMuted", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedHardMute(JSONObject jsonObject) {
            try {
                sendEvent("onReceivedHardMute", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onHardUnMuted(JSONObject jsonObject) {
            try {
                sendEvent("onHardUnMuted", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedHardUnMute(JSONObject jsonObject) {
            try {
                sendEvent("onReceivedHardUnMute", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxLockRoomManagementObserver for sending lock/unlock callbacks events
     */
    private final EnxLockRoomManagementObserver mLockRoomObserver = new EnxLockRoomManagementObserver() {
        @Override
        public void onAckLockRoom(JSONObject jsonObject) {
            try {
                sendEvent("onAckLockRoom", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onAckUnLockRoom(JSONObject jsonObject) {
            try {
                sendEvent("onAckUnLockRoom", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onLockedRoom(JSONObject jsonObject) {
            try {
                sendEvent("onLockedRoom", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onUnLockedRoom(JSONObject jsonObject) {
            try {
                sendEvent("onUnLockedRoom", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxOutBoundCallObserver for sending outbound calls callbacks events
     */
    private final EnxOutBoundCallObserver mEnxOutBoundCallObserver = new EnxOutBoundCallObserver() {
        @Override
        public void onOutBoundCallInitiated(JSONObject jsonObject) {
            try {
                sendEvent("onOutBoundCallInitiated", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onDialStateEvents(EnxRoom.EnxOutBoundCallState enxOutBoundCallState) {
            try {
                HashMap<String, Object> map = new HashMap<>();
                map.put("state", enxOutBoundCallState.toString());
                sendEvent("onDialStateEvents", map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxMuteAudioStreamObserver for sending mute/ummute audiostream  callbacks events
     */
    private final EnxMuteAudioStreamObserver mEnxMuteAudioStreamObserver = new EnxMuteAudioStreamObserver() {
        @Override
        public void onHardMutedAudio(JSONObject jsonObject) {
            try {
                sendEvent("onHardMutedAudio", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onHardUnMutedAudio(JSONObject jsonObject) {
            try {
                sendEvent("onHardUnMutedAudio", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedHardMuteAudio(JSONObject jsonObject) {
            try {
                sendEvent("onReceivedHardMuteAudio", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedHardUnMuteAudio(JSONObject jsonObject) {
            try {
                sendEvent("onOutBoundCallInitiated", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxMuteVideoStreamObserver for sending mute/ummute videostream  callbacks events
     */
    private final EnxMuteVideoStreamObserver mEnxMuteVideoStreamObserver = new EnxMuteVideoStreamObserver() {
        @Override
        public void onHardMutedVideo(JSONObject jsonObject) {
            try {
                sendEvent("onHardMutedVideo", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onHardUnMutedVideo(JSONObject jsonObject) {
            try {
                sendEvent("onHardUnMutedVideo", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedHardMuteVideo(JSONObject jsonObject) {
            try {
                sendEvent("onReceivedHardMuteVideo", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedHardUnMuteVideo(JSONObject jsonObject) {
            try {
                sendEvent("onReceivedHardUnMuteVideo", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxReconnectObserver for sending reconnect callbacks events
     */
    private final EnxReconnectObserver mEnxReconnectObserver = new EnxReconnectObserver() {
        @Override
        public void onReconnect(String s) {
            try {
                HashMap<String, Object> map = new HashMap<>();
                map.put("msg", s);
                sendEvent("onReconnect", map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onUserReconnectSuccess(EnxRoom enxRoom, JSONObject jsonObject) {
            try {
                sendEvent("onUserReconnectSuccess", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxScreenShareObserver for sending screenshare callbacks events
     */
    private final EnxScreenShareObserver mEnxScreenShareObserver = new EnxScreenShareObserver() {
        @Override
        public void onScreenSharedStarted(JSONObject jsonObject) {
            try {
                sendEvent("onScreenSharedStarted", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onScreenSharedStopped(JSONObject jsonObject) {
            try {
                sendEvent("onScreenSharedStopped", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onScreenSharedStarted(EnxStream enxStream) {

        }

        @Override
        public void onScreenSharedStopped(EnxStream enxStream) {

        }
    };

    /**
     * EnxCanvasObserver for sending canvas callbacks events
     */
    private final EnxCanvasObserver mEnxCanvasObserver = new EnxCanvasObserver() {
        @Override
        public void onCanvasStarted(JSONObject jsonObject) {
            try {
                sendEvent("onCanvasStarted", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onCanvasStarted(EnxStream enxStream) {

        }

        @Override
        public void onCanvasStopped(JSONObject jsonObject) {
            try {
                sendEvent("onCanvasStopped", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onCanvasStopped(EnxStream enxStream) {

        }

        @Override
        public void onStartCanvasAck(JSONObject jsonObject) {

        }

        @Override
        public void onStoppedCanvasAck(JSONObject jsonObject) {

        }
    };

    /**
     * EnxFileShareObserver for sending fileshare callbacks events
     */
    private final EnxFileShareObserver mEnxFileShareObserver = new EnxFileShareObserver() {
        @Override
        public void onFileUploadStarted(JSONObject jsonObject) {
            try {
                sendEvent("onFileUploadStarted", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFileAvailable(JSONObject jsonObject) {
            try {
                sendEvent("onFileAvailable", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onInitFileUpload(JSONObject jsonObject) {
            try {
                sendEvent("onInitFileUpload", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFileUploaded(JSONObject jsonObject) {
            try {
                sendEvent("onFileUploaded", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFileUploadCancelled(JSONObject jsonObject) {
            try {
                sendEvent("onFileUploadCancelled", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFileUploadFailed(JSONObject jsonObject) {
            try {
                sendEvent("onFileUploadFailed", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFileDownloaded(JSONObject jsonObject) {
            try {
                sendEvent("onFileDownloaded", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFileDownloadCancelled(JSONObject jsonObject) {
            try {
                sendEvent("onFileDownloadCancelled", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onFileDownloadFailed(JSONObject jsonObject) {
            try {
                sendEvent("onFileDownloadFailed", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onInitFileDownload(JSONObject jsonObject) {
            try {
                sendEvent("onInitFileDownload", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * EnxStatsObserver for sending stats callbacks events
     */
    private final EnxStatsObserver mEnxStatsObserver = new EnxStatsObserver() {
        @Override
        public void onAcknowledgeStats(JSONObject jsonObject) {
            try {
                sendEvent("onAcknowledgeStats", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onReceivedStats(JSONObject jsonObject) {
            try {
                sendEvent("onReceivedStats", jsonToMap(jsonObject));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    };

    private void sendEvent(final String eventName, final HashMap map) {
        map.put("event", eventName);
        mEventHandler.post(new Runnable() {
            @Override
            public void run() {
                if (sink != null) {
                    sink.success(map);
                }
            }
        });
    }


    private JSONObject getAdvancedOptionsObject(ArrayList<Object> advanceOptions) throws JSONException {
        //        [{"battery_updates":false},{"notify_video_resolution_change":false}]
        JSONObject object = new JSONObject();
        for (int i = 0; i < advanceOptions.size(); i++) {
            if (((HashMap) advanceOptions.get(i)).containsKey("battery_updates")) {
                object.put("battery_updates", ((HashMap) advanceOptions.get(i)).get("battery_updates"));
            } else if (((HashMap) advanceOptions.get(i)).containsKey("notify_video_resolution_change")) {
                object.put("notify_video_resolution_change", ((HashMap) advanceOptions.get(i)).get("notify_video_resolution_change"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("options", object);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return jsonObject;

    }

    public JSONObject getRoomInfo() {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("allow_reconnect", true);
            jsonObject.put("number_of_attempts", 3);
            jsonObject.put("timeout_interval", 15);

            JSONObject object = new JSONObject();
            object.put("activeviews", "list");//view
            object.put("audiomute", true);
            object.put("videomute", false);
            object.put("bandwidth", false);
            object.put("screenshot", false);
            object.put("avatar", true);

            object.put("iconColor", mActivity.getResources().getColor(R.color.colorAccent));
            object.put("iconHeight", 30);
            object.put("iconWidth", 30);
            object.put("avatarHeight", 200);
            object.put("avatarWidth", 200);
            jsonObject.put("playerConfiguration", object);
            jsonObject.put("chat_only", false);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return jsonObject;
    }

}
