#import "EnxFlutterPlugin.h"
#import <EnxRTCiOS/EnxRtc.h>
#import <EnxRTCiOS/EnxUtilityManager.h>
#if __has_include(<enx_flutter_plugin/enx_flutter_plugin-Swift.h>)
#import <enx_flutter_plugin/enx_flutter_plugin-Swift.h>
#else
// Support project import fallback if the generated compatibility header
// is not copied when this plugin is created as a library.
// https://forums.swift.org/t/swift-static-libraries-dont-copy-generated-objective-c-header/19816
#import "enx_flutter_plugin-Swift.h"
#endif

@interface EnxPlayer ()
@property(nonatomic, strong) UIView *renderView;
@property(nonatomic, assign) int64_t viewId;
@end

@implementation EnxPlayer
- (instancetype)initWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId {
    if (self = [super init]) {
        self.renderView = [[UIView alloc] initWithFrame:frame];
        self.renderView.backgroundColor = [UIColor redColor];
        self.viewId = viewId;
    }
    return self;
}

- (nonnull UIView *)view {
    return self.renderView;
}

@end

@interface EnxRenderViewFactory : NSObject <FlutterPlatformViewFactory>
@end



@interface EnxFlutterPlugin () <FlutterStreamHandler,EnxRoomDelegate,EnxStreamDelegate,EnxPlayerDelegate>
@property(strong,nonatomic) FlutterMethodChannel *methodChannel;
@property(strong,nonatomic) FlutterEventChannel *eventChannel;
@property(strong,nonatomic) FlutterEventSink eventSink;
@property(strong,nonatomic) EnxRoom *enxRoom;
@property(strong,nonatomic) EnxRtc *enxRTc;
@property(strong,nonatomic) EnxStream *localStream;
@property(strong, nonatomic) NSMutableDictionary *rendererViews;
@property(strong, nonatomic) NSMutableDictionary *playerViews;
@property(strong, nonatomic) NSMutableDictionary *streamDictionary;

@end

@implementation EnxFlutterPlugin
#pragma mark - renderer views

+ (instancetype)sharedPlugin {
    static EnxFlutterPlugin *plugin = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        plugin = [[EnxFlutterPlugin alloc] init];
    });
    return plugin;
}

- (NSMutableDictionary *)rendererViews {
    if (!_rendererViews) {
        _rendererViews = [[NSMutableDictionary alloc] init];
    }
    return _rendererViews;
}

- (NSMutableDictionary *)playerViews {
    if (!_playerViews) {
        _playerViews = [[NSMutableDictionary alloc] init];
    }
    return _playerViews;
}

-(NSMutableDictionary *)streamDictionary{
    if (!_streamDictionary) {
        _streamDictionary = [[NSMutableDictionary alloc] init];
    }
    return _streamDictionary;
}

+ (void)addView:(UIView *)view id:(NSNumber *)viewId {
    if (!viewId) {
        return;
    }
    if (view) {
        [[[EnxFlutterPlugin sharedPlugin] rendererViews] setObject:view forKey:viewId];
    } else {
        [self removeViewForId:viewId];
    }
}

+ (void)removeViewForId:(NSNumber *)viewId {
    if (!viewId) {
        return;
    }
    [[[EnxFlutterPlugin sharedPlugin] rendererViews] removeObjectForKey:viewId];
}

+ (UIView *)viewForId:(NSNumber *)viewId {
    if (!viewId) {
        return nil;
    }
    return [[[EnxFlutterPlugin sharedPlugin] rendererViews] objectForKey:viewId];
}


+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  //[SwiftEnxFlutterPlugin registerWithRegistrar:registrar];


  //[SwiftEnxFlutterPlugin registerWithRegistrar:registrar];
    
       FlutterMethodChannel *channel = [FlutterMethodChannel
               methodChannelWithName:@"enx_flutter_plugin"
                     binaryMessenger:[registrar messenger]];
       FlutterEventChannel *eventChannel = [FlutterEventChannel
               eventChannelWithName:@"enx_flutter_plugin_event_channel"
                    binaryMessenger:registrar.messenger];
       EnxFlutterPlugin *instance = [[EnxFlutterPlugin alloc] init];
       instance.methodChannel = channel;
       instance.eventChannel = eventChannel;
       [registrar addMethodCallDelegate:instance channel:channel];
       
     EnxRenderViewFactory *fac = [[EnxRenderViewFactory alloc] init];
     [registrar registerViewFactory:fac withId:@"EnxPlayer"];
    
}


- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString *method = call.method;
       NSDictionary *params = call.arguments;
       NSLog(@"plugin handleMethodCall: %@, argus: %@", method, params);
       // Core Methods
       if ([@"joinRoom" isEqualToString:method]) {
           NSString *token = [self stringFromArguments:params key:@"token"];
           NSDictionary *publishStreamInfo = [self dictionaryFromArguments:params key:@"localInfo"];
           NSDictionary *roomInfo = [self dictionaryFromArguments:params key:@"roomInfo"];
           NSArray *advanceOptions =[self arrayFromArguments:params key:@"advanceOptions"];
           if(_enxRTc == nil){
               _enxRTc = [[EnxRtc alloc] init];
           }
           _localStream = [_enxRTc joinRoom:token delegate:self PublishStreamInfo:publishStreamInfo roomInfo:roomInfo advanceOptions:advanceOptions];
           _localStream.delegate = self;
           [_eventChannel setStreamHandler:self];
           result(nil);
          }
       else if ([@"publish" isEqualToString:method]){
        if(_enxRoom != nil){
         [_enxRoom publish:_localStream];
         EnxPlayerView *localPlayer = (EnxPlayerView *)[[[EnxFlutterPlugin sharedPlugin] playerViews] valueForKey:@"0"];
         if(localPlayer != nil){
            [_localStream attachRenderer:localPlayer];
           }
         }
         result(nil);
       }
       else if ([@"subscribe" isEqualToString:method]){
         if(params[@"streamId"] != nil){
           EnxStream *remoteStream = [[[EnxFlutterPlugin sharedPlugin] streamDictionary] objectForKey:params[@"streamId"]];
           if(remoteStream != nil){
               [_enxRoom subscribe:remoteStream];
           }
         }
         result(nil);
       }
       else if ([@"disconnect" isEqualToString:method]){
        if(_enxRoom != nil){
           [_enxRoom disconnect];
         }
        result(nil);
       }
       else if ([@"startRecord" isEqualToString:method]){
        if(_enxRoom != nil){
           [_enxRoom startRecord];
         }
        result(nil);
       }
       else if ([@"stopRecord" isEqualToString:method]){
        if(_enxRoom != nil){
           [_enxRoom stopRecord];
         }
        result(nil);
       }
       else if ([@"setTalkerCount" isEqualToString:method]){
        if(_enxRoom != nil){
        NSInteger count = [self intFromArguments:params key:@"count"];
           [_enxRoom setTalkerCount:count];
         }
        result(nil);
       }
       else if ([@"getTalkerCount" isEqualToString:method]){
        if(_enxRoom != nil){
           [_enxRoom getTalkerCount];
         }
        result(nil);
       }
       else if ([@"getMaxTalkers" isEqualToString:method]){
        if(_enxRoom != nil){
           [_enxRoom getMaxTalkers];
         }
        result(nil);
       }
       else if ([@"enableLogs" isEqualToString:method]){
        if(_enxRoom != nil){
           EnxUtilityManager *logger = [EnxUtilityManager shareInstance];
           [logger startLog];
         }
        result(nil);
       }
       else if ([@"postClientLogs" isEqualToString:method]){
        if(_enxRoom != nil){
             [_enxRoom postClientLogs];
           }
        result(nil);
       }
       else if ([@"requestFloor" isEqualToString:method]){
           if(_enxRoom != nil){
             [_enxRoom requestFloor];
           }
        result(nil);
       }
       else if ([@"grantFloor" isEqualToString:method]){
        if(_enxRoom != nil){
            NSString *clintId = [self stringFromArguments:params key:@"clinetId"];
             [_enxRoom grantFloor:clintId];
         }
        result(nil);
       }
       else if ([@"denyFloor" isEqualToString:method]){
        if(_enxRoom != nil){
            NSString *clintId = [self stringFromArguments:params key:@"clinetId"];
             [_enxRoom denyFloor:clintId];
         }
        result(nil);
       }
       else if ([@"releaseFloor" isEqualToString:method]){
        if(_enxRoom != nil){
            NSString *clintId = [self stringFromArguments:params key:@"clinetId"];
             [_enxRoom releaseFloor:clintId];
         }
        result(nil);
       }
       else if ([@"hardMute" isEqualToString:method]){
        if(_enxRoom != nil){
             [_enxRoom hardMute];
         }
        result(nil);
       }
       else if ([@"hardUnMute" isEqualToString:method]){
        if(_enxRoom != nil){
             [_enxRoom hardUnMute];
         }
        result(nil);
       }
       else if ([@"muteSelfAudio" isEqualToString:method]){
        if(_enxRoom != nil){
         if(_localStream != nil){
          BOOL value = [self boolFromArguments:params key:@"isMute"];
             [_localStream muteSelfAudio:value];
          }
         }
        result(nil);
       }
       else if ([@"muteSelfVideo" isEqualToString:method]){
        if(_enxRoom != nil){
         if(_localStream != nil){
          BOOL value = [self boolFromArguments:params key:@"isMute"];
             [_localStream muteSelfVideo:value];
          }
         }
        result(nil);
       }
       else if ([@"hardMuteAudio" isEqualToString:method]){
        if(_enxRoom != nil){
         if(_localStream != nil){
             NSString *clintId = [self stringFromArguments:params key:@"clinetId"];
             [_localStream hardMuteAudio:clintId];
          }
         }
        result(nil);
       }
       else if ([@"hardUnMuteAudio" isEqualToString:method]){
        if(_enxRoom != nil){
         if(_localStream != nil){
             NSString *clintId = [self stringFromArguments:params key:@"clinetId"];
             [_localStream hardUnMuteAudio:clintId];
          }
         }
        result(nil);
       }
       else if ([@"hardMuteVideo" isEqualToString:method]){
        if(_enxRoom != nil){
         if(_localStream != nil){
             NSString *clintId = [self stringFromArguments:params key:@"clinetId"];
             [_localStream hardMuteVideo:clintId];
          }
         }
        result(nil);
       }
       else if ([@"hardUnMuteVideo" isEqualToString:method]){
        if(_enxRoom != nil){
         if(_localStream != nil){
             NSString *clintId = [self stringFromArguments:params key:@"clinetId"];
             [_localStream hardUnMuteVideo:clintId];
          }
         }
        result(nil);
       }
       else if ([@"switchCamera" isEqualToString:method]){
         if(_localStream != nil){
             [_localStream switchCamera];
          }
        result(nil);
       }
       else if ([@"getAdvancedOptions" isEqualToString:method]){
         if(_enxRoom != nil){
             [_enxRoom getAdvanceOptions];
          }
        result(nil);
       }
       else if ([@"lockRoom" isEqualToString:method]){
         if(_enxRoom != nil){
             [_enxRoom lockRoom];
          }
        result(nil);
       }
       else if ([@"unLockRoom" isEqualToString:method]){
         if(_enxRoom != nil){
             [_enxRoom unlockRoom];
          }
        result(nil);
       }
       else if ([@"makeOutboundCall" isEqualToString:method]){
         if(_enxRoom != nil){
             NSString *number = [self stringFromArguments:params key:@"number"];
             [_enxRoom makeOutboundCall:number];
          }
        result(nil);
       }
       else if ([@"extendConferenceDuration" isEqualToString:method]){
         if(_enxRoom != nil){
             [_enxRoom extendConferenceDuration];
          }
        result(nil);
       }
       else if ([@"destroy" isEqualToString:method]){
         if(_enxRoom != nil){
             [_enxRoom destroy];
          }
        result(nil);
       }
       else if ([@"stopVideoTracksOnApplicationBackground" isEqualToString:method]){
         if(_enxRoom != nil){
          BOOL value = [self boolFromArguments:params key:@"videoMuteRemoteStream"];
           [_enxRoom stopVideoTracksOnApplicationBackground:value];
          }
        result(nil);
       }
       else if ([@"startVideoTracksOnApplicationForeground" isEqualToString:method]){
         if(_enxRoom != nil){
          BOOL value = [self boolFromArguments:params key:@"restoreVideoRemoteStream"];
           [_enxRoom startVideoTracksOnApplicationForeground:value];
          }
        result(nil);
       }
       else if ([@"setAudioOnlyMode" isEqualToString:method]){
         if(_enxRoom != nil){
          BOOL value = [self boolFromArguments:params key:@"audioOnly"];
           [_enxRoom setAudioOnlyMode:value];
          }
        result(nil);
       }
       else if ([@"setAudioOnlyMode" isEqualToString:method]){
         if(_enxRoom != nil){
          BOOL value = [self boolFromArguments:params key:@"audioOnly"];
           [_enxRoom setAudioOnlyMode:value];
          }
        result(nil);
       }
       else if ([@"switchMediaDevice" isEqualToString:method]){
         if(_enxRoom != nil){
          NSString *deviceName = [self stringFromArguments:params key:@"deviceName"];
             [_enxRoom switchMediaDevice:deviceName];
          }
        result(nil);
       }
       else if ([@"switchUserRole" isEqualToString:method]){
         if(_enxRoom != nil){
          NSString *clientId = [self stringFromArguments:params key:@"clientId"];
             [_enxRoom switchUserRole:clientId];
          }
        result(nil);
       }
       else if ([@"enableStats" isEqualToString:method]){
         if(_enxRoom != nil){
          BOOL value = [self boolFromArguments:params key:@"enableStats"];
             [_enxRoom enableStats:value];
          }
        result(nil);
       }
       else if ([@"muteSubscribeStreamsAudio" isEqualToString:method]){
         if(_enxRoom != nil){
          BOOL value = [self boolFromArguments:params key:@"isMute"];
             [_enxRoom muteSubscribeStreamsAudio:value];
          }
        result(nil);
       }
       else if ([@"sendMessage" isEqualToString:method]){
         if(_enxRoom != nil){
          NSString *message = [self stringFromArguments:params key:@"message"];
          BOOL isBroadCast = [self boolFromArguments:params key:@"isBroadCast"];
          NSArray *recipientIDs = [self arrayFromArguments:params key:@"recipientIDs"];
          [_enxRoom sendMessage:message isBroadCast:isBroadCast recipientIDs:recipientIDs];
         }
        result(nil);
       }
       else if ([@"sendUserData" isEqualToString:method]){
         if(_enxRoom != nil){
          NSDictionary *message = [self dictionaryFromArguments:params key:@"message"];
          BOOL isBroadCast = [self boolFromArguments:params key:@"isBroadCast"];
          NSArray *recipientIDs = [self arrayFromArguments:params key:@"recipientIDs"];
          [_enxRoom sendUserData:message isBroadCast:isBroadCast recipientIDs:recipientIDs];
         }
        result(nil);
       }
       else if ([@"sendFiles" isEqualToString:method]){
         if(_enxRoom != nil){
          NSString *position = [self stringFromArguments:params key:@"position"];
          BOOL isBroadCast = [self boolFromArguments:params key:@"isBroadCast"];
          NSArray *clientIds = [self arrayFromArguments:params key:@"clientIds"];
             EnxFilePosition pos = Top;
             if(position != nil){
                position = [position uppercaseString];
              if ([position isEqualToString:@"BOTTOM"]){
                  pos = Bottom;
              }
              else if ([position isEqualToString:@"CENTER"]){
                  pos = Center;
              }
             }
             [_enxRoom sendFiles:isBroadCast clientIds:clientIds];
         }
        result(nil);
       }
       else if ([@"downloadFile" isEqualToString:method]){
         if(_enxRoom != nil){
          NSDictionary *file = [self dictionaryFromArguments:params key:@"file"];
          BOOL autoSave = [self boolFromArguments:params key:@"autoSave"];
          [_enxRoom downloadFile:file autoSave:autoSave];
         }
        result(nil);
       }
       else if ([@"cancelUpload" isEqualToString:method]){
         if(_enxRoom != nil){
          NSInteger jobId = [self intFromArguments:params key:@"jobId"];
          [_enxRoom cancelUpload:jobId];
         }
        result(nil);
       }
       else if ([@"cancelAllUploads" isEqualToString:method]){
         if(_enxRoom != nil){
          [_enxRoom cancelAllUploads];
         }
        result(nil);
       }
       else if ([@"cancelDownload" isEqualToString:method]){
         if(_enxRoom != nil){
          NSInteger jobId = [self intFromArguments:params key:@"jobId"];
          [_enxRoom cancelDownload:jobId];
         }
        result(nil);
       }
       else if ([@"cancelAllDownloads" isEqualToString:method]){
         if(_enxRoom != nil){
          [_enxRoom cancelAllDownloads];
         }
        result(nil);
       }
       else if ([@"dropUser" isEqualToString:method]){
         if(_enxRoom != nil){
          NSArray *clientIds = [self arrayFromArguments:params key:@"clientIds"];
          [_enxRoom dropUser:clientIds];
         }
        result(nil);
       }
       else if ([@"getUserList" isEqualToString:method]){
           NSArray *userList;
        if(_enxRoom != nil){
          userList = [_enxRoom getUserList];
         }
        result(userList);
       }
       else if ([@"getReceiveVideoQuality" isEqualToString:method]){
           NSString *quality;
        if(_enxRoom != nil){
          NSString *streamType = [self stringFromArguments:params key:@"streamType"];
          quality = [_enxRoom getReceiveVideoQuality:streamType];
         }
        result(quality);
       }
       else if ([@"Whoami" isEqualToString:method]){
           NSDictionary *whoami;
        if(_enxRoom != nil){
          whoami = [_enxRoom Whoami];
         }
        result(whoami);
       }
       else if ([@"getDevices" isEqualToString:method]){
           NSArray *getDevices;
        if(_enxRoom != nil){
          getDevices = [_enxRoom getDevices];
         }
        result(getDevices);
       }
       else if ([@"getSelectedDevice" isEqualToString:method]){
           NSString *getSelectedDevice;
        if(_enxRoom != nil){
          getSelectedDevice = [_enxRoom getSelectedDevice];
         }
        result(getSelectedDevice);
       }
       else if ([@"getAvailableFiles" isEqualToString:method]){
           NSArray *getAvailableFiles;
        if(_enxRoom != nil){
          getAvailableFiles = [_enxRoom getAvailableFiles];
         }
        result(getAvailableFiles);
       }
       else if ([@"getRoomId" isEqualToString:method]){
           NSString *roomId = @"";
        if(_enxRoom != nil){
          roomId = _enxRoom.roomId;
         }
        result(roomId);
       }
       else if ([@"getClientName" isEqualToString:method]){
           NSString *clientName = @"";
        if(_enxRoom != nil){
          clientName = _enxRoom.clientName;
         }
        result(clientName);
       }
       else if ([@"getRole" isEqualToString:method]){
           NSString *getRole = @"";
        if(_enxRoom != nil){
          getRole = _enxRoom.userRole;
         }
        result(getRole);
       }
       else if ([@"getClientId" isEqualToString:method]){
           NSString *clientId = @"";
        if(_enxRoom != nil){
          clientId = _enxRoom.clientId;
         }
        result(clientId);
       }
       else if ([@"isRoomActiveTalker" isEqualToString:method]){
           BOOL isRoomActiveTalker = false;
        if(_enxRoom != nil){
          isRoomActiveTalker = _enxRoom.isRoomActiveTalker;
         }
        result([NSNumber numberWithBool:isRoomActiveTalker]);
       }
       else if ([@"cancelFloor" isEqualToString:method]){
        if(_enxRoom != nil){
           [_enxRoom cancelFloor];
         }
        result(nil);
       }
       else if ([@"finishFloor" isEqualToString:method]){
        if(_enxRoom != nil){
           [_enxRoom finishFloor];
         }
        result(nil);
       }
       else if ([@"enableProximitySensor" isEqualToString:method]){
        if(_enxRoom != nil){
             BOOL value = [self boolFromArguments:params key:@"isEnabled"];
           [_enxRoom enableProximitySensor:value];
         }
        result(nil);
       }
       else if ([@"config" isEqualToString:method]){
        if(_enxRoom != nil){
             NSDictionary *value = [self dictionaryFromArguments:params key:@"config"];
           [_enxRoom updateConfiguration:value];
         }
        result(nil);
       }
    else if ([@"setupVideo" isEqualToString:method]) {
        NSInteger viewId = [self intFromArguments:params key:@"viewId"];
        UIView *view = [EnxFlutterPlugin viewForId:@(viewId)];
        float height = 0.0;
        float width = 0.0;
        if(params[@"height"] != nil){
            height = [params[@"height"] floatValue];
        }
        if(params[@"width"] != nil){
            width = [params[@"width"] floatValue];
        }
        if(params[@"uid"] == nil){
            return;
        }
        if([params[@"isLocal"] boolValue] == true){
            EnxPlayerView *localView =(EnxPlayerView *)[[[EnxFlutterPlugin sharedPlugin] playerViews] valueForKey:[params[@"uid"]stringValue]];
            if(localView != nil){
              [localView removeFromSuperview];
            }
            else{
             localView = [[EnxPlayerView alloc] initLocalView:CGRectMake(0, 0, width, height)];
             localView.delegate = self;
            }
            localView.clipsToBounds = true;
             [view addSubview:localView];
           [[[EnxFlutterPlugin sharedPlugin] playerViews] setObject:localView forKey:[params[@"uid"]stringValue]];
        }
        else {
         EnxPlayerView *remoteView =(EnxPlayerView *)[[[EnxFlutterPlugin sharedPlugin] playerViews] valueForKey:[params[@"uid"]stringValue]];
         if(remoteView != nil){
             [remoteView removeFromSuperview];
         }
         else{
           remoteView = [[EnxPlayerView alloc] initRemoteView:CGRectMake(0, 0, width, height)];
           remoteView.delegate = self;
         }
          remoteView.clipsToBounds = true;
          [view addSubview:remoteView];
         [[[EnxFlutterPlugin sharedPlugin] playerViews] setObject:remoteView forKey:[params[@"uid"] stringValue]];
          EnxStream *stream =(EnxStream *)[[[EnxFlutterPlugin sharedPlugin] streamDictionary] valueForKey:[params[@"uid"]stringValue]];
            if(stream != nil){
                [stream attachRenderer:remoteView];
            }
        }
        result(nil);
    }
    else if ([@"captureScreenShot" isEqualToString:method]){
      if(_enxRoom != nil){
       NSArray *streamId = [self stringFromArguments:params key:@"streamId"];
       EnxPlayerView *playerView =(EnxPlayerView *)[[[EnxFlutterPlugin sharedPlugin] playerViews] valueForKey:streamId];
          if (playerView != nil){
              [playerView captureScreenShot];
          }
      }
     result(nil);
    }
    else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)dealloc {
    [self.methodChannel setMethodCallHandler:nil];
    [self.eventChannel setStreamHandler:nil];
}

#pragma mark - FlutterStreamHandler

- (FlutterError *_Nullable)onCancelWithArguments:(id _Nullable)params {
    _eventSink = nil;
    return nil;
}

- (FlutterError *_Nullable)onListenWithArguments:(id _Nullable)params eventSink:(nonnull FlutterEventSink)events {
    _eventSink = events;
    return nil;
}


- (void)sendEvent:(NSString *)name params:(NSDictionary *)params {
    if (_eventSink) {
        NSMutableDictionary *dict = [[NSMutableDictionary alloc] initWithDictionary:params];
        dict[@"event"] = name;
        _eventSink([dict copy]);
    }
}


#pragma mark - helper

- (NSString *)stringFromArguments:(NSDictionary *)params key:(NSString *)key {
    if (![params isKindOfClass:[NSDictionary class]]) {
        return nil;
    }

    NSString *value = [params valueForKey:key];
    if (![value isKindOfClass:[NSString class]]) {
        return nil;
    } else {
        return value;
    }
}

- (NSDictionary *)dictionaryFromArguments:(NSDictionary *)params key:(NSString *)key {
    if (![params isKindOfClass:[NSDictionary class]]) {
        return nil;
    }

    NSDictionary *value = [params valueForKey:key];
    if (![value isKindOfClass:[NSDictionary class]]) {
        return nil;
    } else {
        return value;
    }
}

- (NSArray *)arrayFromArguments:(NSDictionary *)params key:(NSString *)key {
    if (![params isKindOfClass:[NSDictionary class]]) {
        return nil;
    }

    NSArray *value = [params valueForKey:key];
    if (![value isKindOfClass:[NSArray class]]) {
        return nil;
    } else {
        return value;
    }
}

- (NSInteger)intFromArguments:(NSDictionary *)params key:(NSString *)key {
    if (![params isKindOfClass:[NSDictionary class]]) {
        return 0;
    }

    NSNumber *value = [params valueForKey:key];
    if (![value isKindOfClass:[NSNumber class]]) {
        return 0;
    } else {
        return [value integerValue];
    }
}

- (BOOL)boolFromArguments:(NSDictionary *)params key:(NSString *)key {
    if (![params isKindOfClass:[NSDictionary class]]) {
        return NO;
    }

    NSNumber *value = [params valueForKey:key];
    if (![value isKindOfClass:[NSNumber class]]) {
        return NO;
    } else {
        return [value boolValue];
    }
}


#pragma mark - EnxRTCiOS_Room Delegate

//Mark - EnxRoom Delegates
/*
 This Delegate will notify to User Once he got succes full join Room
 */

-(void)room:(EnxRoom *)room didConnect:(NSDictionary *)roomMetadata{
    NSLog(@"connectedddd");
    _enxRoom = room;
    [self sendEvent:@"onRoomConnected" params:roomMetadata];
}

/*
 This Delegate will notify to User Once he Getting error in joining room
 */
- (void)room:(EnxRoom *_Nullable)room didError:(NSArray *_Nullable)reason {
    NSLog(@"roomErrrorrr:  %@",reason);
    if(reason == nil || reason.count == 0){
        return;
    }
    [self sendEvent:@"onRoomError" params:reason[0]];
}

-(void)room:(EnxRoom *)room didEventError:(NSArray *)reason{
     NSLog(@"eventErrrorrr:  %@",reason);
    if(reason == nil || reason.count == 0){
         return;
     }
     [self sendEvent:@"onEventError" params:reason[0]];
}

-(void)room:(EnxRoom *)room didEventInfo:(NSDictionary *)infoData{
    if(infoData == nil){
        return;
    }
    [self sendEvent:@"onEventInfo" params:infoData];
}

-(void)room:(EnxRoom *)room didPublishStream:(EnxStream *)stream{
    
}

-(void)didRoomDisconnect:(NSArray *)response{
    if(response == nil || response.count == 0){
        return;
    }
    [self sendEvent:@"onRoomDisConnected" params:response[0]];
}

-(void)room:(EnxRoom *)room userDidDisconnected:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onUserDisConnected" params:Data[0]];
}

-(void)room:(EnxRoom *)room userDidJoined:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onUserConnected" params:Data[0]];
}

-(void)room:(EnxRoom *)room didUserDataReceived:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onUserDataReceived" params:data[0]];
}

-(void)room:(EnxRoom *)room didMessageReceived:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onMessageReceived" params:data[0]];
}

-(void)room:(EnxRoom *)room didAddedStream:(EnxStream *)stream{
    
    if(stream == nil){
        return;
    }
    [[[EnxFlutterPlugin sharedPlugin] streamDictionary] setValue:stream forKey:stream.streamId];
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    if (stream.streamId != nil){
        [dict setValue:stream.streamId forKey:@"streamId"];
    }
    [dict setValue:[NSNumber numberWithBool:stream.hasData] forKey:@"hasData"];
    [dict setValue:[NSNumber numberWithBool:stream.screen] forKey:@"hasScreen"];
    [self sendEvent:@"onStreamAdded" params:dict];
}

-(void)room:(EnxRoom *)room didActiveTalkerList:(NSArray *)Data{
   @try{
    if(Data == nil || Data.count == 0){
        return;
    }
    NSMutableArray *array = [[NSMutableArray alloc] init];
    for (int i = 0; i < Data.count; i++) {
     EnxStream *stream = (EnxStream *)Data[i];
        NSString *streamId = @"";NSString *clientId = @"";
        NSString *mediatype = @"";NSString *name = @"";
        NSString *reason = @"";NSString *videoaspectratio = @"";
        if(stream.streamId != nil){
            streamId = stream.streamId;
        }
        if(stream.clientId != nil){
            clientId = stream.clientId;
        }
        if(stream.mediaType != nil){
            mediatype = stream.mediaType;
        }
        if(stream.name != nil){
            name = stream.name;
        }
        if(stream.videoAspectRatio != nil){
            videoaspectratio = stream.videoAspectRatio;
        }
        if(stream.reasonForMuteVideo != nil){
            reason = stream.reasonForMuteVideo;
        }
        NSDictionary *tempDict = @{@"streamId":streamId,@"clientId":clientId,@"mediatype":mediatype,@"name":name,@"reason":reason,@"videomuted":[NSNumber numberWithBool:stream.isSelfVideoMuted],@"videoaspectratio":videoaspectratio};
     [array insertObject:tempDict atIndex:i];
     EnxPlayerView *player = (EnxPlayerView *)[[[EnxFlutterPlugin sharedPlugin] playerViews] objectForKey:stream.streamId];
     if(stream != nil && player != nil){
         [stream attachRenderer:player];
     }
    }
    [self sendEvent:@"onActiveTalkerList" params:@{@"activeList":array}];
   }
   @catch(NSException *error){
       NSLog(@"%@",error.description);
    }
}

//-(void)room:(EnxRoom *)room activeTalkerList:(NSArray *)Data {
//    if(Data == nil || Data.count == 0){
//        return;
//    }
//    NSDictionary *dataDict = Data[0];
//    [self sendEvent:@"onActiveTalkerList" params:dataDict];
//    NSArray *activeListArray = dataDict[@"activeList"];
//    for (int i = 0; i < activeListArray.count; i++) {
//        NSDictionary *activeListDict = activeListArray[i];
//        NSString *streamId = [NSString stringWithFormat:@"%@",activeListDict[@"streamId"]];
//        EnxStream *stream = (EnxStream *)_enxRoom.streamsByStreamId[streamId];
//        EnxPlayerView *player = (EnxPlayerView *)[[[EnxFlutterPlugin sharedPlugin] playerViews] objectForKey:streamId];
//        if(stream != nil && player != nil) {
//            [stream attachRenderer:player];
//        }
//    }
//}

//-(void)room:(EnxRoom *)room didActiveTalkerList:(NSArray *)Data{
//
//}

-(void)room:(EnxRoom *)room didSwitchUserRole:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onSwitchedUserRole" params:data[0]];
}

-(void)room:(EnxRoom *)room didAckDestroy:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onAckDestroy" params:data[0]];
}

-(void)room:(EnxRoom *)room didAckDropUser:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onAckDropUser" params:data[0]];
}

-(void)room:(EnxRoom *)room didConferenceRemainingDuration:(NSArray *)data{
   if(data == nil || data.count == 0){
       return;
   }
   [self sendEvent:@"onConferenceRemainingDuration" params:data[0]];
}

-(void)room:(EnxRoom *)room didConferencessExtended:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onConferencessExtended" params:data[0]];
}

-(void)roomRecordOn:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onRoomRecordingOn" params:Data[0]];
}

-(void)startRecordingEvent:(NSArray *)response{
    if(response == nil || response.count == 0){
        return;
    }
    [self sendEvent:@"onStartRecordingEvent" params:response[0]];
}

-(void)roomRecordOff:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onRoomRecordingOff" params:Data[0]];
}

-(void)stopRecordingEvent:(NSArray *)response{
    if(response == nil || response.count == 0){
        return;
    }
    [self sendEvent:@"onStopRecordingEvent" params:response[0]];
}

-(void)room:(EnxRoom *)room didSetTalkerCount:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onSetTalkerCount" params:Data[0]];
}

-(void)room:(EnxRoom *)room didGetTalkerCount:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onGetTalkerCount" params:Data[0]];
}

-(void)room:(EnxRoom *)room didGetMaxTalkers:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onMaxTalkerCount" params:Data[0]];
}

-(void)didLogUpload:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onLogUploaded" params:data[0]];
}

-(void)room:(EnxRoom *)room didBandWidthUpdated:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onBandWidthUpdated" params:data[0]];
}

-(void)room:(EnxRoom *)room didShareStreamEvent:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onShareStreamEvent" params:data[0]];
}

-(void)room:(EnxRoom *)room didCanvasStreamEvent:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onCanvasStreamEvent" params:data[0]];
}

-(void)room:(EnxRoom *)room didConnectionInterrupted:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onConnectionInterrupted" params:data[0]];
}

-(void)room:(EnxRoom *)room didConnectionLost:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onConnectionLost" params:data[0]];
}

-(void)didFloorRequested:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onFloorRequested" params:Data[0]];
}

-(void)didFloorRequestReceived:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onFloorRequestReceived" params:Data[0]];
}

-(void)didProcessFloorRequested:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onProcessFloorRequested" params:Data[0]];
}

-(void)didGrantFloorRequested:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onGrantedFloorRequest" params:Data[0]];
}

-(void)didDenyFloorRequested:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onDeniedFloorRequest" params:Data[0]];
}

-(void)didReleaseFloorRequested:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onReleasedFloorRequest" params:Data[0]];
}

-(void)didHardMuteReceived:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onReceivedHardMute" params:Data[0]];
}

-(void)didhardMute:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onHardMuted" params:Data[0]];
}

-(void)didHardunMuteReceived:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onReceivedHardUnMute" params:Data[0]];
}

-(void)didhardUnMute:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onHardUnMuted" params:Data[0]];
}



-(void)room:(EnxRoom *)room didGetAdvanceOptions:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onGetAdvancedOptions" params:data[0]];
}

-(void)room:(EnxRoom *)room didAckLockRoom:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onAckLockRoom" params:data[0]];
}

-(void)room:(EnxRoom *)room didAckUnlockRoom:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onAckUnLockRoom" params:data[0]];
}

-(void)room:(EnxRoom *)room didLockRoom:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onLockedRoom" params:data[0]];
}

-(void)room:(EnxRoom *)room didUnlockRoom:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onUnLockedRoom" params:data[0]];
}

-(void)room:(EnxRoom *)room didOutBoundCallInitiated:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onOutBoundCallInitiated" params:data[0]];
}

-(void)room:(EnxRoom *)room didDialStateEvents:(EnxOutBoundCallState)state{
    NSDictionary *dict;
    if (state == Initiated){
        dict = @{@"state":@"Initiated"};
    }
    else if (state == Ringing){
        dict = @{@"state":@"Ringing"};
    }
    else if (state == Connected){
        dict = @{@"state":@"Connected"};
    }
    else if (state == Failed){
        dict = @{@"state":@"Failed"};
    }
    else{
        dict = @{@"state":@"Disconnected"};
    }
    [self sendEvent:@"onDialStateEvents" params:dict];

}

-(void)room:(EnxRoom *)room didReconnect:(NSString *)reason{
    if(reason == nil){
        return;
    }
    [self sendEvent:@"onReconnect" params:@{@"msg":reason}];
}

-(void)room:(EnxRoom *)room didUserReconnectSuccess:(NSDictionary *)data{
    if(data == nil){
        return;
    }
    [self sendEvent:@"onUserReconnectSuccess" params:data];
}

-(void)didNotifyDeviceUpdate:(NSString *)updates{
    if(updates == nil){
        return;
    }
    NSDictionary *dataDict = @{@"msg":updates};
    [self sendEvent:@"onNotifyDeviceUpdate" params:dataDict];
}


-(void)room:(EnxRoom *)room screenSharedStarted:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onScreenSharedStarted" params:Data[0]];
}

-(void)room:(EnxRoom *)room screenShareStopped:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onScreenSharedStopped" params:Data[0]];
}

-(void)room:(EnxRoom *)room canvasStarted:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onCanvasStarted" params:Data[0]];
}

-(void)room:(EnxRoom *)room canvasStopped:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onCanvasStopped" params:Data[0]];
}

-(void)didReceiveStats:(NSArray *)statsData{
    if(statsData == nil || statsData.count == 0){
           return;
    }
    [self sendEvent:@"onReceiveStats" params:statsData[0]];
}

-(void)room:(EnxRoom *)room didAcknowledgSendData:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onAckSendData" params:data[0]];
}

-(void)room:(EnxRoom *)room didFileUploadStarted:(NSArray *)data{
     if(data == nil || data.count == 0){
            return;
     }
     [self sendEvent:@"onFileUploadStarted" params:data[0]];
}

-(void)room:(EnxRoom *)room didFileAvailable:(NSArray *)data{
     if(data == nil || data.count == 0){
            return;
     }
     [self sendEvent:@"onFileAvailable" params:data[0]];
}

-(void)room:(EnxRoom *)room didInitFileUpload:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onInitFileUpload" params:data[0]];
}

-(void)room:(EnxRoom *)room didFileUploaded:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onFileUploaded" params:data[0]];
}

-(void)room:(EnxRoom *)room didFileUploadCancelled:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onFileUploadCancelled" params:data[0]];
}

-(void)room:(EnxRoom *)room didFileUploadFailed:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onFileUploadFailed" params:data[0]];
}

-(void)room:(EnxRoom *)room didFileDownloaded:(NSString *)data{
    if(data == nil){
           return;
    }
    [self sendEvent:@"onFileDownloaded" params:@{@"msg":data}];
}

-(void)room:(EnxRoom *)room didFileDownloadCancelled:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onFileDownloadCancelled" params:data[0]];
}

-(void)room:(EnxRoom *)room didFileDownloadFailed:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onFileDownloadFailed" params:data[0]];
}

-(void)room:(EnxRoom *)room didInitFileDownload:(NSArray *)data{
    if(data == nil || data.count == 0){
           return;
    }
    [self sendEvent:@"onInitFileDownload" params:data[0]];
}

-(void)didCancelledFloorRequest:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
           return;
    }
    [self sendEvent:@"onCancelledFloorRequest" params:Data[0]];
}

-(void)didFloorCancelled:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
           return;
    }
    [self sendEvent:@"onFloorCancelled" params:Data[0]];
}

-(void)didFinishedFloorRequest:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
           return;
    }
    [self sendEvent:@"onFinishedFloorRequest" params:Data[0]];
}

-(void)didFloorFinished:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
           return;
    }
    [self sendEvent:@"onFloorFinished" params:Data[0]];
}

#pragma mark - EnxRTCiOS_Stream Delegate

-(void)didAudioEvents:(NSDictionary *)data{
    if(data == nil){
        return;
    }
    [self sendEvent:@"onAudioEvent" params:data];
}

-(void)didVideoEvents:(NSDictionary *)data{
    if(data == nil){
        return;
    }
    [self sendEvent:@"onVideoEvent" params:data];
}

-(void)didReceiveData:(NSDictionary *)data{
    if(data == nil){
        return;
    }
    [self sendEvent:@"onReceivedData" params:data];
}

-(void)stream:(EnxStream *)stream didRemoteStreamAudioMute:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onRemoteStreamAudioMute" params:data[0]];
}

-(void)stream:(EnxStream *)stream didRemoteStreamAudioUnMute:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onRemoteStreamAudioUnMute" params:data[0]];
}

-(void)stream:(EnxStream *)stream didRemoteStreamVideoMute:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onRemoteStreamVideoMute" params:data[0]];
}

-(void)stream:(EnxStream *)stream didRemoteStreamVideoUnMute:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onRemoteStreamVideoUnMute" params:data[0]];
}

-(void)didhardMuteAudio:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onHardMutedAudio" params:Data[0]];
}

-(void)didhardUnMuteAudio:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onHardUnMutedAudio" params:Data[0]];
}

-(void)didRecievedHardMutedAudio:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onReceivedHardMuteAudio" params:Data[0]];
}

-(void)didRecievedHardUnmutedAudio:(NSArray *)Data{
    if(Data == nil || Data.count == 0){
        return;
    }
    [self sendEvent:@"onReceivedHardUnMuteAudio" params:Data[0]];
}

-(void)stream:(EnxStream *)stream didHardVideoMute:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onHardMutedVideo" params:data[0]];
}

-(void)stream:(EnxStream *)stream didHardVideoUnMute:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onHardUnMutedVideo" params:data[0]];
}

-(void)stream:(EnxStream *)stream didReceivehardMuteVideo:(NSArray *)data{
    if(data == nil || data.count == 0){
        return;
    }
    [self sendEvent:@"onReceivedHardMuteVideo" params:data[0]];
}

-(void)stream:(EnxStream *)stream didRecivehardUnmuteVideo:(NSArray *)data{
     if(data == nil || data.count == 0){
         return;
     }
     [self sendEvent:@"onReceivedHardUnMuteVideo" params:data[0]];
}

//Player Delegate
-(void)didCapturedView:(UIImage *)snapShot{
    if(snapShot != nil){
     NSData *imageData = UIImagePNGRepresentation(snapShot);
     NSString * base64String = [imageData base64EncodedStringWithOptions:0];
     if(base64String != nil){
         [self sendEvent:@"OnCapturedView" params:@{@"bitmap":base64String}];
     }
    }
}

@end

@implementation EnxRenderViewFactory
- (nonnull NSObject <FlutterPlatformView> *)createWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id _Nullable)args {
    EnxPlayer *rendererView = [[EnxPlayer alloc] initWithFrame:frame viewIdentifier:viewId];
  //  [rendererView.view setBackgroundColor:[UIColor redColor]];
    [EnxFlutterPlugin addView:rendererView.view id:@(viewId)];
    return rendererView;
}

@end
