import 'package:enx_flutter_plugin/enx_flutter_plugin.dart';
import 'package:flutter/material.dart';
import 'base.dart';
// import 'base.dart';

/// EnxPlayerWidget - This widget will automatically manage the native view.
///
/// Enables create native view with `uid` `mode` `local` and destroy native view automatically.
///
class EnxPlayerWidget extends StatefulWidget {
  // uid
  final int uid;
  final int height;
  final int width;

  // local flag
  final bool local;
  final ScalingType mScalingType;
  final bool zMediaOverlay;

  EnxPlayerWidget(
    this.uid, {
    this.local = false,
    this.height = 200,
    this.width = 300,
    this.zMediaOverlay = false,
    this.mScalingType = ScalingType.SCALE_ASPECT_FIT,
    Key key,
  })  : assert(uid != null),
        assert(local != null),
        assert(mScalingType != null),
        assert(zMediaOverlay != null),
        assert(height != null),
        assert(width != null),
        super(key: key ?? Key(uid.toString()));

  @override
  State<StatefulWidget> createState() => _EnxPlayerWidgetState();
}

class _EnxPlayerWidgetState extends State<EnxPlayerWidget> {
  Widget _nativeView;

  int _viewId;

  @override
  void initState() {
    super.initState();
    _nativeView = EnxFlutterPlugin.createNativeView((viewId) {
      print('rashinativeView' + viewId.toString());
      _viewId = viewId;
      _bindView();
    });
  }

  @override
  void dispose() {
    EnxFlutterPlugin.removeNativeView(_viewId);
    super.dispose();
  }

  @override
  void didUpdateWidget(EnxPlayerWidget oldWidget) {
    super.didUpdateWidget(oldWidget);

    if ((widget.uid != oldWidget.uid || widget.local != oldWidget.local) &&
        _viewId != null) {
      _bindView();
      return;
    }

    if (widget.mScalingType != oldWidget.mScalingType) {
      setScalingType();
      return;
    }
    if (widget.zMediaOverlay != oldWidget.zMediaOverlay) {
      setScalingType();
      return;
    }
  }

  void _bindView() {
    if (widget.local) {
      EnxFlutterPlugin.setupVideo(
          _viewId, 0, widget.local, widget.width, widget.height);
    } else {
      EnxFlutterPlugin.setupVideo(
          _viewId, widget.uid, widget.local, widget.width, widget.height);
    }
  }

  void setScalingType() {
    if (widget.local) {
      EnxFlutterPlugin.setPlayerScalingType(
          widget.mScalingType, _viewId, 0, widget.local);
    } else {
      EnxFlutterPlugin.setPlayerScalingType(
          widget.mScalingType, _viewId, widget.uid, widget.local);
    }
  }

  void setMediaOverlay() {
    if (widget.local) {
      EnxFlutterPlugin.setZOrderMediaOverlay(_viewId, 0, widget.zMediaOverlay);
    } else {
      EnxFlutterPlugin.setZOrderMediaOverlay(_viewId, widget.uid, widget.local);
    }
  }

  @override
  Widget build(BuildContext context) => _nativeView;
}
